Imports FarPoint.Win.Spread
Imports System.Drawing
Imports System.Windows.Forms
Public Class z9iFps : Inherits FarPoint.Win.Spread.FpSpread

    Public Enum ColumnDataType
        stdGeneral
        sdtString
        sdtInteger
        sdtDecimal
        sdtMoney
        sdtDate
        sdtCustom
        sdtCheckBox
        sdtImage
    End Enum

    '=========================================
    ' member variables
    Private m_AllowSorting As Boolean = True

    Private m_CurrentSortDirection As Windows.Forms.SortOrder
    Private SearchWhileTyping As String
    Private m_SortIgnoreFirstN As Integer = -1
    Private m_SortIgnoreLastN As Integer = -1

    Private m_CurrentCell As Integer
    '===========================================
    ' Printing
    Private m_PrintLandscape As Boolean = True
    Private m_PrintTitle As String = ""
    Private m_PrintFooter As String = ""
    Private m_PrintLeftMargin As Single = 20

    Private m_PrintingLine As Integer
    Private m_PrintingPageNo As Integer
    Private m_PrintingCurrentY As Single
    Private m_PrintingTotalWidth As Integer
    Private m_PrintingNewPage As Boolean
    Private m_PrintingRowPos As Integer
    Private m_PrintingUserName As String

    Private m_PrintingoStringFormat As StringFormat
    Private m_PrintingoStringFormatComboBox As StringFormat
    Private m_PrintingoButton As New Button
    Private m_PrintingoCheckbox As New CheckBox
    Friend WithEvents ctxGridMenu As System.Windows.Forms.ContextMenuStrip
    Private components As System.ComponentModel.IContainer
    Friend WithEvents CopyToolStripMenuItem As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents SelectAllToolStripMenuItem As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents PrintToolStripMenuItem As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents FreezePanesToolStripMenuItem As System.Windows.Forms.ToolStripMenuItem
    Friend WithEvents CopyAllToolStripMenuItem As System.Windows.Forms.ToolStripMenuItem
    Private m_PrintingoComboBox As New ComboBox


    Public Sub New()
        MyBase.New()
        InitializeComponent()
    End Sub


    '==========================================
    ' Add a column to the grid
    Public Function AddGridColumn(ByVal SheetNo As Integer, _
                                    ByVal Name As String, _
                                    ByVal Header As String, _
                                    ByVal DataType As ColumnDataType, _
                                    ByVal AllowSort As Boolean, _
                                    ByVal Width As Integer, _
                                    ByVal TextAlign As Windows.Forms.HorizontalAlignment, _
                                    ByVal ForeColor As Color, _
                                    ByVal BackColor As Color, _
                                    ByVal Allowedit As Boolean, _
                                    ByVal Frozen As Boolean, _
                                    ByVal FormatString As String) As FarPoint.Win.Spread.Column


        Dim sh As SheetView = Me.Sheets(SheetNo)
        Dim i As Integer


        sh.ColumnCount += 1
        i = sh.ColumnCount - 1


        sh.ColumnHeader.Columns(i).Width = Width
        sh.ColumnHeader.Cells(0, i).Text = Header
        sh.ColumnHeader.Columns(i).ShowSortIndicator = AllowSort

        Select Case DataType
            Case ColumnDataType.stdGeneral
                sh.Columns(i).CellType = New FarPoint.Win.Spread.CellType.GeneralCellType
            Case ColumnDataType.sdtInteger
                Dim intType As New FarPoint.Win.Spread.CellType.NumberCellType
                intType.DecimalPlaces = 0
                intType.Separator = ","
                intType.ShowSeparator = True
                sh.Columns(i).CellType = intType
            Case ColumnDataType.sdtDecimal
                Dim decType As New FarPoint.Win.Spread.CellType.NumberCellType
                decType.DecimalPlaces = 2
                decType.Separator = ","
                decType.ShowSeparator = True
                sh.Columns(i).CellType = decType
            Case ColumnDataType.sdtMoney
                Dim curType As New FarPoint.Win.Spread.CellType.CurrencyCellType
                curType.CurrencySymbol = ""
                curType.ShowCurrencySymbol = False
                curType.DecimalPlaces = 2
                curType.Separator = ","
                curType.ShowSeparator = True
                curType.DecimalSeparator = "."
                sh.Columns(i).CellType = curType
            Case ColumnDataType.sdtDate
                Dim datType As New FarPoint.Win.Spread.CellType.DateTimeCellType
                datType.DateTimeFormat = "dd/MMM/yyyy"
                sh.Columns(i).CellType = datType
            Case ColumnDataType.sdtString, ColumnDataType.sdtCustom
                Dim genType As New FarPoint.Win.Spread.CellType.GeneralCellType
                sh.Columns(i).CellType = genType
            Case ColumnDataType.sdtCheckBox
                Dim chkType As New FarPoint.Win.Spread.CellType.CheckBoxCellType
                sh.Columns(i).CellType = chkType
            Case ColumnDataType.sdtImage
                Dim imgType As New FarPoint.Win.Spread.CellType.ImageCellType()
                sh.Columns(i).CellType = imgType

        End Select

        sh.Columns(i).AllowAutoSort = AllowSort
        sh.Columns(i).BackColor = BackColor
        sh.Columns(i).ForeColor = ForeColor

        Select Case TextAlign
            Case HorizontalAlignment.Center
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Center
            Case HorizontalAlignment.Left
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Left
            Case HorizontalAlignment.Right
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Right
            Case Else
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.General
        End Select

        ' TODO Format
        ' sh.Columns(i).Formatter

        sh.Columns(i).Locked = Not Allowedit
        If Frozen Then
            sh.FrozenColumnCount = i
        End If
        Return sh.Columns(i)
    End Function
    '==========================================
    ' Add a column to the grid
    Public Function AddGridColumn(ByVal SheetNo As Integer, _
                                    ByVal Name As String, _
                                    ByVal Header As String, _
                                    ByVal DataType As ColumnDataType, _
                                    ByVal AllowSort As Boolean, _
                                    ByVal Width As Integer, _
                                    ByVal TextAlign As Windows.Forms.HorizontalAlignment, _
                                    Optional ByVal AllowEdit As Boolean = False, _
                                    Optional ByVal Frozen As Boolean = False, _
                                    Optional ByVal FormatString As String = "") As FarPoint.Win.Spread.Column


        Dim sh As SheetView = Me.Sheets(SheetNo)
        Dim i As Integer


        sh.ColumnCount += 1
        i = sh.ColumnCount - 1


        sh.ColumnHeader.Columns(i).Width = Width
        sh.ColumnHeader.Cells(0, i).Text = Header
        sh.ColumnHeader.Columns(i).ShowSortIndicator = AllowSort

        Select Case DataType
            Case ColumnDataType.stdGeneral
                sh.Columns(i).CellType = New FarPoint.Win.Spread.CellType.GeneralCellType
            Case ColumnDataType.sdtInteger
                Dim intType As New FarPoint.Win.Spread.CellType.NumberCellType
                intType.DecimalPlaces = 0
                intType.Separator = ","
                intType.ShowSeparator = True
                sh.Columns(i).CellType = intType
            Case ColumnDataType.sdtDecimal
                Dim decType As New FarPoint.Win.Spread.CellType.NumberCellType
                decType.DecimalPlaces = 2
                decType.Separator = ","
                decType.ShowSeparator = True
                sh.Columns(i).CellType = decType
            Case ColumnDataType.sdtMoney
                Dim curType As New FarPoint.Win.Spread.CellType.CurrencyCellType
                curType.CurrencySymbol = ""
                curType.ShowCurrencySymbol = False
                curType.DecimalPlaces = 2
                curType.Separator = ","
                curType.ShowSeparator = True
                curType.DecimalSeparator = "."
                sh.Columns(i).CellType = curType
            Case ColumnDataType.sdtDate
                Dim datType As New FarPoint.Win.Spread.CellType.DateTimeCellType
                datType.DateTimeFormat = CellType.DateTimeFormat.UserDefined
                datType.DateSeparator = "/"
                datType.UserDefinedFormat = "dd/MM/yyyy"
                sh.Columns(i).CellType = datType
            Case ColumnDataType.sdtString, ColumnDataType.sdtCustom
                Dim genType As New FarPoint.Win.Spread.CellType.GeneralCellType
                sh.Columns(i).CellType = genType
            Case ColumnDataType.sdtCheckBox
                Dim chkType As New FarPoint.Win.Spread.CellType.CheckBoxCellType
                sh.Columns(i).CellType = chkType
            Case ColumnDataType.sdtImage
                Dim imgType As New FarPoint.Win.Spread.CellType.ImageCellType()
                sh.Columns(i).CellType = imgType
        End Select


        sh.Columns(i).AllowAutoSort = AllowSort

        Select Case TextAlign
            Case HorizontalAlignment.Center
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Center
            Case HorizontalAlignment.Left
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Left
            Case HorizontalAlignment.Right
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.Right
            Case Else
                sh.Columns(i).HorizontalAlignment = CellHorizontalAlignment.General
        End Select

        ' TODO Format
        ' sh.Columns(i).Formatter

        sh.Columns(i).Locked = Not AllowEdit
        If Frozen Then
            sh.FrozenColumnCount = i
        End If
        Return sh.Columns(i)
    End Function
    '=====================================================
    ' Add row
    Public Function AddGridRow(ByVal SheetNo As Integer, ByVal FirstColumnValue As Object) As Integer
        Dim i As Integer
        Me.Sheets(SheetNo).Rows.Count += 1
        i = Me.Sheets(SheetNo).Rows.Count - 1
        Me.Sheets(SheetNo).Cells(i, 0).Value = FirstColumnValue
        Return i
    End Function
    '=====================================================
    ' Insert row
    Public Function InsertGridRow(ByVal SheetNo As Integer, ByVal Index As Integer, ByVal FirstColumnValue As String) As Integer
        Me.Sheets(SheetNo).Rows.Add(Index, 1)
        Me.Sheets(SheetNo).Cells(Index, 0).Value = FirstColumnValue
        Return Index
    End Function

    '=======================================
    ' Get or Set the value of the cell specifed by row and col parameters
    Public Property ValueMatrix(ByVal SheetNo As Integer, ByVal Row As Integer, ByVal Col As Integer) As Object
        Get
            If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                Return Me.Sheets(SheetNo).Cells(Row, Col).Value
            Else
                Return ""
            End If
        End Get
        Set(ByVal Value As Object)
            Try
                If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                    Me.Sheets(SheetNo).Cells(Row, Col).Value = Value
                End If
            Catch
                ' ignore
            End Try
        End Set
    End Property
    '=======================================
    ' Get or Set the value of the cell specifed by row and col parameters
    Public Property TextMatrix(ByVal SheetNo As Integer, ByVal Row As Integer, ByVal Col As Integer) As String
        Get
            If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                Return Me.Sheets(SheetNo).Cells(Row, Col).Text
            Else
                Return ""
            End If
        End Get
        Set(ByVal Value As String)
            Try
                If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                    Me.Sheets(SheetNo).Cells(Row, Col).Text = Value
                End If
            Catch
                ' ignore
            End Try
        End Set
    End Property
    '=======================================
    ' Get or Set the backcolor of the cell specifed by row and col parameters
    Public Property BackColorMatrix(ByVal SheetNo As Integer, ByVal Row As Integer, ByVal Col As Integer) As Color
        Get
            If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                Return Me.Sheets(SheetNo).Cells(Row, Col).BackColor
            Else
                Return Nothing
            End If
        End Get
        Set(ByVal Value As Color)
            Try
                If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                    Me.Sheets(SheetNo).Cells(Row, Col).BackColor = Value
                End If
            Catch
                ' ignore
            End Try
        End Set
    End Property

    '=======================================
    ' Get or Set the forecolor of the cell specifed by row and col parameters
    Public Property ForeColorMatrix(ByVal SheetNo As Integer, ByVal Row As Integer, ByVal Col As Integer) As Color
        Get
            If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                Return Me.Sheets(SheetNo).Cells(Row, Col).ForeColor
            Else
                Return Nothing
            End If
        End Get
        Set(ByVal Value As Color)
            Try
                If Row < Me.Sheets(SheetNo).Rows.Count And Col < Me.Sheets(SheetNo).Columns.Count Then
                    Me.Sheets(SheetNo).Cells(Row, Col).ForeColor = Value
                End If
            Catch
                ' ignore
            End Try
        End Set
    End Property


    Public ReadOnly Property FPSpread() As FarPoint.Win.Spread.FpSpread
        Get
            Return Me
        End Get
    End Property

    '===========================================
    ' Grid level flag for allow sorting
    Public Property AllowSorting() As Boolean
        Get
            Return m_AllowSorting
        End Get
        Set(ByVal Value As Boolean)
            m_AllowSorting = Value
        End Set
    End Property

    '==========================================
    ' When sorting do not include First N rows in the sort
    Public Property SortIgnoreFirstN() As Integer
        Get
            Return m_SortIgnoreFirstN
        End Get
        Set(ByVal Value As Integer)
            m_SortIgnoreFirstN = Value
        End Set
    End Property

    '==========================================
    ' When sorting do not include Last N rows in the sort
    Public Property SortIgnoreLastN() As Integer
        Get
            Return m_SortIgnoreLastN
        End Get
        Set(ByVal Value As Integer)
            m_SortIgnoreLastN = Value
        End Set
    End Property
    '=========================================
    ' Printing properties
    Public Property PrintLandscape() As Boolean
        Get
            Return m_PrintLandscape
        End Get
        Set(ByVal Value As Boolean)
            m_PrintLandscape = Value
        End Set
    End Property
    Public Property PrintLeftMargin() As Single
        Get
            Return m_PrintLeftMargin
        End Get
        Set(ByVal Value As Single)
            m_PrintLeftMargin = Value
        End Set
    End Property

    Public Property PrintTitle() As String
        Get
            Return m_PrintTitle
        End Get
        Set(ByVal Value As String)
            m_PrintTitle = Value
        End Set
    End Property
    Public Property PrintFooter() As String
        Get
            Return m_PrintFooter
        End Get
        Set(ByVal Value As String)
            m_PrintFooter = Value
        End Set
    End Property

    Public Property UserName() As String
        Get
            Return m_PrintingUserName
        End Get
        Set(ByVal value As String)
            m_PrintingUserName = value
        End Set
    End Property
    ''' <summary>
    ''' Clear all rows and columns by calling base class clear and then remove all z9Grid internal columns.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Clear(ByVal SheetNo As Integer)
        Me.Sheets(SheetNo).Rows.Count = 0
        Me.Sheets(SheetNo).Columns.Count = 0
    End Sub

    Public ReadOnly Property DefaultContextMenu() As ContextMenuStrip
        Get
            Return ctxGridMenu
        End Get
    End Property
    ''===============================================
    '' Implement search while typing
    'Private Sub z9Grid_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Me.KeyPress
    '    ' 
    '    If m_CurrentSortDirection <> Windows.Forms.SortOrder.None Then
    '        Dim lvi As DataGridViewRow
    '        If Char.IsDigit(e.KeyChar) Or Char.IsLetter(e.KeyChar) Then
    '            Static LastTimeHere As Double = Microsoft.VisualBasic.Timer()
    '            If Microsoft.VisualBasic.Timer() - LastTimeHere > 3 Then
    '                SearchWhileTyping = ""
    '            End If
    '            e.Handled = True
    '            SearchWhileTyping += Char.ToUpper(e.KeyChar)
    '            If SearchWhileTyping <> "" Then
    '                For Each lvi In Me.Rows
    '                    If Not lvi.IsNewRow Then
    '                        If Strings.Left(lvi.Cells(Me.SortedColumn.Name).Value.ToString().ToUpper(), Len(SearchWhileTyping)) = SearchWhileTyping Then
    '                            Me.ClearSelection()
    '                            lvi.Selected = True
    '                            ' me.EnsureVisible
    '                            Exit For
    '                        End If
    '                    End If
    '                Next
    '            End If
    '            LastTimeHere = Microsoft.VisualBasic.Timer()
    '        Else
    '            SearchWhileTyping = ""
    '        End If
    '    End If
    'End Sub

    ''=================================================
    '' Enforce the minimum column width. 
    'Public Sub EnsureColumnMinWidth(Optional ByVal ColumnPosition As Integer = -1)
    '    Dim cl As DataGridViewColumn
    '    Dim j As Integer
    '    If ColumnPosition = -1 Then
    '        For Each cl In Me.Columns
    '            If Me.Columns(j).Width < cl.MinimumWidth Then
    '                Me.Columns(j).Width = cl.MinimumWidth
    '            End If
    '            j += 1
    '        Next
    '    Else
    '        If Me.Columns(ColumnPosition).Width < CType(Me.Columns(ColumnPosition), DataGridViewColumn).MinimumWidth Then
    '            Me.Columns(j).Width = CType(Me.Columns(ColumnPosition), DataGridViewColumn).MinimumWidth
    '        End If
    '    End If
    'End Sub

    '===============================================
    ' Built-in context menu event handlers
    Private Sub ToolStripMenuItemPrint_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles PrintToolStripMenuItem.Click
        Dim pi As New PrintInfo()
        pi.Header = m_PrintTitle
        pi.JobName = "eCube - Print Grid"
        pi.Orientation = PrintOrientation.Auto
        pi.PageOrder = PrintPageOrder.Auto
        pi.ShowPrintDialog = True
        pi.PrintNotes = PrintNotes.AtEnd
        pi.ShowGrid = True
        pi.ShowRowHeader = True
        pi.ShowColumnHeader = True
        pi.Preview = True
        pi.Orientation = PrintOrientation.Auto
        Me.ActiveSheet.PrintInfo = pi
        Me.PrintSheet(Me.ActiveSheet)
    End Sub

    Private Sub ToolStripMenuItemSelectAll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles SelectAllToolStripMenuItem.Click
        Me.ActiveSheet.Models.Selection.SetSelection(-1, -1, Me.ActiveSheet.RowCount, Me.ActiveSheet.ColumnCount)
    End Sub
    Private Sub ToolStripFreezePanes_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles FreezePanesToolStripMenuItem.Click
        Dim cr As FarPoint.Win.Spread.Model.CellRange = Me.ActiveSheet.Models.Selection(0)
        'Dim s As String = m_fps.ActiveSheet.GetClip(cr.Row, cr.Column, cr.RowCount, cr.ColumnCount)
        Me.ActiveSheet.FrozenRowCount = cr.Row
        Me.ActiveSheet.FrozenColumnCount = cr.Column
    End Sub
    Public Sub MergeDefaultMenu(ByVal CtxMenuStrip As ContextMenuStrip)
        CtxMenuStrip.Items.Add(CopyToolStripMenuItem)
        CtxMenuStrip.Items.Add(CopyAllToolStripMenuItem)
        CtxMenuStrip.Items.Add(SelectAllToolStripMenuItem)
        CtxMenuStrip.Items.Add(PrintToolStripMenuItem)
        CtxMenuStrip.Items.Add(FreezePanesToolStripMenuItem)
    End Sub


    ''=============================================
    '' Resize all columns
    'Public Sub ResizeAllColumns()
    '    Dim i As Integer
    '    For i = 0 To Me.Columns.Count - 1
    '        If Me.Columns(i).Width > 0 Then
    '            Me.Columns(i).Width = -1
    '        End If
    '    Next
    'End Sub


    Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Me.ctxGridMenu = New System.Windows.Forms.ContextMenuStrip(Me.components)
        Me.CopyToolStripMenuItem = New System.Windows.Forms.ToolStripMenuItem
        Me.SelectAllToolStripMenuItem = New System.Windows.Forms.ToolStripMenuItem
        Me.PrintToolStripMenuItem = New System.Windows.Forms.ToolStripMenuItem
        Me.FreezePanesToolStripMenuItem = New System.Windows.Forms.ToolStripMenuItem
        Me.CopyAllToolStripMenuItem = New System.Windows.Forms.ToolStripMenuItem
        Me.ctxGridMenu.SuspendLayout()
        CType(Me, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        '
        'ctxGridMenu
        '
        Me.ctxGridMenu.Items.AddRange(New System.Windows.Forms.ToolStripItem() {Me.CopyToolStripMenuItem, Me.CopyAllToolStripMenuItem, Me.SelectAllToolStripMenuItem, Me.PrintToolStripMenuItem, Me.FreezePanesToolStripMenuItem})
        Me.ctxGridMenu.Name = "ctxGridMenu"
        Me.ctxGridMenu.Size = New System.Drawing.Size(143, 92)
        '
        'CopyToolStripMenuItem
        '
        Me.CopyToolStripMenuItem.Name = "CopyToolStripMenuItem"
        Me.CopyToolStripMenuItem.Size = New System.Drawing.Size(142, 22)
        Me.CopyToolStripMenuItem.Text = "Copy"
        '
        'SelectAllToolStripMenuItem
        '
        Me.SelectAllToolStripMenuItem.Name = "SelectAllToolStripMenuItem"
        Me.SelectAllToolStripMenuItem.Size = New System.Drawing.Size(142, 22)
        Me.SelectAllToolStripMenuItem.Text = "Select All"
        '
        'PrintToolStripMenuItem
        '
        Me.PrintToolStripMenuItem.Name = "PrintToolStripMenuItem"
        Me.PrintToolStripMenuItem.Size = New System.Drawing.Size(142, 22)
        Me.PrintToolStripMenuItem.Text = "Print"
        '
        'FreezePanesToolStripMenuItem
        '
        Me.FreezePanesToolStripMenuItem.Name = "FreezePanesToolStripMenuItem"
        Me.FreezePanesToolStripMenuItem.Size = New System.Drawing.Size(142, 22)
        Me.FreezePanesToolStripMenuItem.Text = "Freeze Panes"
        '
        'CopyAllToolStripMenuItem
        '
        Me.CopyAllToolStripMenuItem.Name = "CopyAllToolStripMenuItem"
        Me.CopyAllToolStripMenuItem.Size = New System.Drawing.Size(142, 22)
        Me.CopyAllToolStripMenuItem.Text = "Copy All"
        '
        'z9iFps
        '
        Me.ContextMenuStrip = Me.ctxGridMenu
        Me.ActiveSheetIndex = -1
        Me.ctxGridMenu.ResumeLayout(False)
        CType(Me, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)

    End Sub

    Private Sub CopyToolStripMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles CopyToolStripMenuItem.Click
        Me.ClipboardOptions = FarPoint.Win.Spread.ClipboardOptions.AllHeaders
        Me.ActiveSheet.ClipboardCopyShape()
    End Sub

    Private Sub CopyAllToolStripMenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles CopyAllToolStripMenuItem.Click
        Me.ClipboardOptions = FarPoint.Win.Spread.ClipboardOptions.AllHeaders
        Me.ActiveSheet.ClipboardCopy()
    End Sub
End Class
